
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# constant definitions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

PRONOUNS = \
    [ 'thou', 'thee', 'thine', 'thy', 'i', 'me', 'mine', 'my', \
      'we', 'us', 'ours', 'our', 'you', 'yours', 'your', 'he', \
      'him', 'his', 'she', 'her', 'hers', 'it', 'its', 'they', \
      'them', 'their', 'theirs' ]

STATE_ABBREVIATIONS = \
    [ 'al', 'ak', 'az', 'ar', 'ca', 'co', 'ct', 'de', 'dc', \
      'fl', 'ga', 'hi', 'id', 'il', 'in', 'ia', 'ks', 'ky', \
      'la', 'me', 'md', 'ma', 'mi', 'mn', 'ms', 'mo', 'mt', \
      'ne', 'nv', 'nh', 'nj', 'nm', 'ny', 'nc', 'nd', 'oh', \
      'ok', 'or', 'pa', 'ri', 'sc', 'sd', 'tn', 'tx', 'ut',
      'vt', 'va', 'wa', 'wv', 'wi', 'wy' ]

STATE_CAPITALS = \
    [ 'montgomery', 'juneau', 'phoenix', 'little rock',
      'sacramento', 'denver', 'hartford', 'dover', 'tallahassee',
      'atlanta', 'honolulu', 'boise', 'springfield', 'indianapolis',
      'des moines', 'topeka', 'frankfort', 'baton rouge', 'augusta',
      'annapolis', 'boston', 'lansing', 'st. paul', 'jackson',
      'jefferson city', 'helena', 'lincoln', 'carson city',
      'concord', 'trenton', 'santa fe', 'albany', 'raleigh',
      'bismarck', 'columbus', 'oklahoma city', 'salem',
      'harrisburg', 'providence', 'columbia', 'pierre', 'nashville',
      'austin', 'salt lake city', 'montpelier', 'richmond',
      'olympia', 'charleston', 'madison', 'cheyenne' ]

MONTHS = \
    [ 'january', 'february', 'march', 'april', 'may', 'june', 'july',
      'august', 'september', 'october', 'november', 'december' ]

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# string helper functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def scrub(word):
    return word.strip().lower()

def str_remove(part, string):
    pos = string.find(part)
    if pos == -1:
        return string
    return string[0:pos] + string[pos+len(part):]

def starts_with_pronoun(word):
    for p in PRONOUNS:
        if word.startswith(p):
            return p
    return False

def is_pronoun(word):
    return word in PRONOUNS

def number_of_vowels(word):
    VOWELS = 'aeiouy'
    count = 0
    for c in word:
        if c in VOWELS:
            count += 1
    return count

def find_all_in(desired_chars, word):
    result = ''
    for char in word:
        if char in desired_chars:
            result += char
    return result

def str_contains(word, char_list):
    selected_chars = find_all_in(char_list, word)
    return char_list in selected_chars

def str_contains_repeat(char, str):
    pos = str.find(char+char)
    return pos != -1

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# puzzles definitions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def puzzle_a(word_list):
    count = len(word_list)
    for i in range(count):
        j = i + 1
        while j < count and word_list[j].startswith(word_list[i]):
            if word_list[j] == (word_list[i] + 'er'):
                break
            j += 1
        else:
            continue
        k = j + 1
        while k < count and word_list[k].startswith(word_list[i]):
            if word_list[k] == (word_list[i] + 'est'):
                break
            k += 1
        else:
            continue
        print(word_list[i], word_list[j], word_list[k])

def puzzle_b(word_list):
    target_chars = 'lmnopqrstuv'
    for word in word_list:
        target = 'lmnopqrstuv'
        count  = 0
        for char in word:
            if char in target:
                count += 1
                target = str_remove(char, target)
        if count == len(target_chars)-1:
            print(word, 'contains all but one of', target_chars, \
                  '('+target+')')

def puzzle_c(word_list):
    for word in word_list:
        prefix = starts_with_pronoun(word)
        if prefix and is_pronoun(word[len(prefix):]):
                print(word)

def puzzle_d(word_list):
    for word in word_list:
        if len(word) != 6:
            continue
        if word[0] != 'c':
            continue
        opposite = 'h'+word[1:] 
        if opposite in word_list:
                print(word, opposite)

def puzzle_e(word_list):
    for word in word_list:
        if len(word) == 7              and \
           number_of_vowels(word) == 1 and \
           's' not in word:
            print(word)

def puzzle_f(word_list):
    # similar to b...
    target_chars = 'memphis'
    for word in word_list:
        target = target_chars
        count  = 0
        for char in word:
            if char in target_chars:
                count += 1
                target = str_remove(char, target)
        if count == len(target_chars) and target == '':
            print(word, 'contains all of', target_chars)

def puzzle_g(word_list):
    target_str = 'tantan'
    for word in word_list:
        if target_str in word:
            print(word)

def puzzle_h(word_list):
    for word in word_list:
        i = 0
        while i < len(word)-1:
            if word[i:i+2] not in STATE_ABBREVIATIONS:
                break
            i += 1
        else:
            if len(word) >= 6:
                print(word)

def puzzle_i(word_list):
    # identical to f...
    target_chars = 'ijtx'
    for word in word_list:
        target = target_chars
        count  = 0
        for char in word:
            if char in target_chars:
                count += 1
                target = str_remove(char, target)
        if count == len(target_chars) and target == '':
            print(word, 'contains all of', target_chars)

def puzzle_j(word_list):
    # identical to g...
    target_str = 'nacl'
    for word in word_list:
        if target_str in word:
            print(word)

def puzzle_k(word_list):
    # from Session 19
    for word in word_list:
        if len(word) >= 5 and str_contains(word, 'aeiou'):
            print(word)

def puzzle_l(word_list):
    alphabet = 'abcdefghijklmnopqrstuvwxyz'
    goal_str = 'rate'
    for word in word_list:
        original = word
        if len(word) != 8:
            continue
        for c in alphabet:
            if str_contains_repeat(c, word):
                word = str_remove(c+c, word)
        if word == goal_str:
            print(original)

def puzzle_m(word_list):
    for word in word_list:
        if not word.startswith('sw'):
            continue
        suffix = word[2:]
        if not 'tw' + suffix in word_list:
            continue
        if 'wh' + suffix in word_list:
            print(word, 'tw' + suffix, 'wh' + suffix)

def puzzle_n(word_list):
    for capital in STATE_CAPITALS:
        for month in MONTHS:
            if capital.startswith(month):
                print(capital)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# machinery for the main script
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
def load_word_list(filename):
    word_file = open (filename, 'r')
    word_list = []
    for word in word_file:
        word_list.append( scrub(word) )
    return word_list

def get_puzzle(letter):
    index = PUZZLES.index(letter) + 1
    return PUZZLES[index]

PUZZLES = ['a', puzzle_a, 'b', puzzle_b, 'c', puzzle_c,
           'd', puzzle_d, 'e', puzzle_e, 'f', puzzle_f,
           'g', puzzle_g, 'h', puzzle_h, 'i', puzzle_i,
           'j', puzzle_j, 'k', puzzle_k, 'l', puzzle_l,
           'm', puzzle_m, 'n', puzzle_n]

WORD_FILE = 'dictionary.txt'

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# main script
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
print('Loading word list.')
word_list = load_word_list(WORD_FILE)

while True:
    print()
    print('Enter letter of puzzle to solve, <return> to quit.')
    puzzle_letter= input('puzzle to solve: ')
    if puzzle_letter =='':
        break
    print()
    print('Solving puzzle (' + puzzle_letter + ').')
    puzzle = get_puzzle(puzzle_letter)
    puzzle(word_list)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
